package info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.exc.UnrecognizedPropertyException;
import info.aspirecn.iov.sjjh.commons.lang.ChannelTypeHandleResponseObject;
import info.aspirecn.iov.sjjh.commons.lang.Constant;
import info.aspirecn.iov.sjjh.commons.lang.PatternTools;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.constant.SjjhConstant;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.service.IdentityCardService;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.utils.DESedeUtil;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.utils.DecodeUtil;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.utils.IdentityCardUpApiHttp;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.vo.IdentityCardCheckResponseObject;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.vo.IdentityCardJsonResolveObject;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.vo.TwoElementsRequestObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.util.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import java.io.IOException;
import java.net.SocketTimeoutException;
import java.util.*;


/**
 *
 * @author lixinxu
 *
 */
@Slf4j
@Service
public class IdentityCardServiceImpl implements IdentityCardService {

	@Value("${custom.identityCardUpApi.api}")
	private String identityCardUpApiUri;
	@Value("${custom.identityCardUpApi.apiKey}")
	private String identityCardUpApiKey;
	@Value("${custom.identityCardUpApi.usernaem}")
	private String identityCardUpApiUsernaem;
	@Value("${custom.identityCardUpApi.privateKey}")
	private String identityCardUpApiPrivateKey;
	@Value("${wz.decryptKey}")
	private String decryptKey;
	/**
	 * 身份证二要素验证
	 *
	 * @param request
	 * @param customBody
	 * @param outTime
	 * @return
	 */
	@Override
	public ChannelTypeHandleResponseObject identityCardTwoPointCheck(HttpServletRequest request, String customBody, int outTime, String ifJkCode) {

		ChannelTypeHandleResponseObject ret = new ChannelTypeHandleResponseObject();
		ret.setIsCharge(Constant.INTERFACE_QUERY_NO_FEE);
		String upstreamCode = "" + Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE;
		request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, upstreamCode);
		try {
			if (outTime <= 0) {
				outTime = Constant.HTTPCLIENT_CONNECTTIMEOUT;
			}
			log.info("identityCardTwoPointCheck---customBody={},outTime={},ifJkCode={}", customBody, outTime, ifJkCode);
			ObjectMapper objectMapper = new ObjectMapper();
			IdentityCardJsonResolveObject jsonResolveObject = objectMapper.readValue(customBody,
					IdentityCardJsonResolveObject.class);

			//参数校验
			log.info("supplier10000035.identityCardTwoPointCheck---requestObject={}", jsonResolveObject);
			List<String> paramErrorResultList = new ArrayList<>();
			//"^[\\u4E00-\\u9FA5\\uf900-\\ufa2d\\u3400-\\u4DB5\\u20000-\\u2A6D6·s]{2,30}$";
			//2020-05-03 修改姓名验证
			//姓名
			/*if (StringUtils.isBlank(jsonResolveObject.getName()) || !PatternTools.checkResult(Constant.PATTERN_CN_NAME_REGEX, jsonResolveObject.getName())) {
				paramErrorResultList.add("name");
			}
			if(StringUtils.isBlank(jsonResolveObject.getName())
					|| jsonResolveObject.getName().length() > 5
					|| (!(jsonResolveObject.getName().replaceAll("[\u4e00-\u9fa5]*[A-Z]*\\d*-*_*\\s*", "").length()==0))){
				paramErrorResultList.add("name");
			}*/
			//身份证号
			if (StringUtils.isBlank(jsonResolveObject.getIdentityCard())
					|| jsonResolveObject.getIdentityCard().length() > 18
					|| (!jsonResolveObject.getIdentityCard().matches("\\d{15}|\\d{17}[\\dxX]"))){
				paramErrorResultList.add("identityCard");
			}

			if (paramErrorResultList.size() > 0) {
				ret.setCode(Constant.SUCCESS);
				ret.setIsCharge(Constant.IS_NOT_CHARGE);
				ret.setResultCode(Constant.CUSTOMER_RETURN_JK.equals(ifJkCode) ? Constant.JK_RETURN_CODE_INVALID_PARAMETER : Constant.INVALID_PARAMETER_CODE);
				ret.setResultBody(Constant.INVALID_PARAMETER);
				ret.setResultDesc(Constant.INVALID_PARAMETER);
				return ret;
			}

			//请求的上游接口
			Map<String, String> apiParam = new HashMap<>();
			apiParam.put("name",jsonResolveObject.getName());
			apiParam.put("identityCard",jsonResolveObject.getIdentityCard());

			String responseContext = IdentityCardUpApiHttp.postApi(identityCardUpApiUri, identityCardUpApiKey, identityCardUpApiUsernaem,
					identityCardUpApiPrivateKey, apiParam, outTime);

			log.info("supplier10000035.identityCardTwoPointCheck接口responseContext={}", responseContext);


			IdentityCardCheckResponseObject responseObject = null;
			if(responseContext != null && !"".equals(responseContext)){
				responseObject = objectMapper.readValue(
						responseContext, IdentityCardCheckResponseObject.class);
			}

			log.info("supplier10000035.identityCardTwoPointCheck接口responseObject={}", responseObject);

			if (responseObject != null) {
				ret.setCode(Constant.SUCCESS);
				setTwoElementsCheckResponse(ret, ifJkCode, responseObject);
				request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, responseObject.getCode());
			} else {
				ret.setCode(Constant.REQUEST_TIMEOUT);
			}

		} catch (SocketTimeoutException ste) {
			ErrorUtils.captureException(ste);
			log.info("supplier10000035.identityCardTwoPointCheck接口SocketTimeoutException={}", ste);
			ret.setCode(Constant.REQUEST_TIMEOUT);
		} catch (UnrecognizedPropertyException upe) {
			log.info("supplier10000035.identityCardTwoPointCheck接口-UnrecognizedPropertyException：{}", upe);
			ErrorUtils.captureException(upe);
			ret.setCode(Constant.SUCCESS);
			ret.setIsCharge(Constant.IS_NOT_CHARGE);
			/** 参数错误 */
			ret.setIsCharge(Constant.IS_NOT_CHARGE);
			ret.setResultCode(Constant.CUSTOMER_RETURN_JK.equals(ifJkCode) ? Constant.JK_RETURN_CODE_INVALID_PARAMETER : Constant.INVALID_PARAMETER_CODE);
			ret.setResultBody(Constant.INVALID_PARAMETER);
			ret.setResultDesc(Constant.PARAMETER_NAME_ERROR);
			return ret;
		} catch (IOException ioe) {
			log.info("supplier10000035.identityCardTwoPointCheck接口-Exception：{}", ioe);
			ErrorUtils.captureException(ioe);
			ret.setCode(Constant.FAIL);
		} catch (Exception ioe) {
			log.info("supplier10000035.identityCardTwoPointCheck接口-Exception：{}", ioe);
			ErrorUtils.captureException(ioe);
			ret.setCode(Constant.FAIL);
		}
		return ret;
	}

	public static void main(String[] args) {
		String name ="比拉力丁·加拉力丁丁丁丁";
		if (StringUtils.isBlank(name) || !PatternTools.checkResult(Constant.PATTERN_CN_NAME_REGEX, name)) {
			System.out.println("---------");
		}

	}

	@Override
	public ChannelTypeHandleResponseObject twoElementsCheckWeiZhongDecode(HttpServletRequest request, String customBody, int outTime, String ifJkCode) {
		ChannelTypeHandleResponseObject ret = new ChannelTypeHandleResponseObject();
		ret.setIsCharge(Constant.INTERFACE_QUERY_NO_FEE);
		String upstreamCode = "" + Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE;
		request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, upstreamCode);
		String returnType = request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA);
		if (StringUtils.isBlank(returnType)) {
			returnType = Constant.CUSTOMER_RETURN_ZW;
		}
		if (outTime <= 0) {
			outTime = Constant.HTTPCLIENT_CONNECTTIMEOUT;
		}
		log.info("supplier10000003.twoElementsCheckWeiZhongDecode---customBody={},outTime={}", customBody, outTime);

		customBody = DecodeUtil.decodeParams(ret, returnType, customBody);
		if (StringUtils.isBlank(customBody)) {
			ret.setCode(Constant.SUCCESS);
			request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, upstreamCode);
			return ret;
		}

		try {
			ObjectMapper objectMapper = new ObjectMapper();
			TwoElementsRequestObject requestObject = objectMapper.readValue(customBody,
					TwoElementsRequestObject.class);
			log.info("supplier10000003.twoElementsCheckWeiZhongDecode---requestObject={}", requestObject);

			// 参数校验
			if (!validateRequestParams(ret, returnType, requestObject)) {
				ret.setCode(Constant.SUCCESS);
				request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, upstreamCode);
				return ret;
			}

			//请求的上游接口
			Map<String, String> apiParam = new HashMap<>();
			apiParam.put("name",requestObject.getName());
			apiParam.put("identityCard",requestObject.getIdCode());

			String responseContext = IdentityCardUpApiHttp.postApi(identityCardUpApiUri, identityCardUpApiKey, identityCardUpApiUsernaem,
					identityCardUpApiPrivateKey, apiParam, outTime);

			IdentityCardCheckResponseObject responseObject = null;
			if(responseContext != null && !"".equals(responseContext)){
				responseObject = objectMapper.readValue(
						responseContext, IdentityCardCheckResponseObject.class);
			}
			if (responseObject != null) {
				ret.setCode(Constant.SUCCESS);
				setTwoElementsCheckResponse(ret, ifJkCode, responseObject);
				request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, responseObject.getCode());
			} else {
				ret.setCode(Constant.REQUEST_TIMEOUT);
			}
		} catch (SocketTimeoutException ste) {
			log.info("supplier10000003.twoElementsCheckWeiZhongDecode接口-SocketTimeoutException：{}", ste);
			ErrorUtils.captureException(ste);
			ret.setCode(Constant.REQUEST_TIMEOUT);
		} catch (Exception ioe) {
			log.info("supplier10000003.twoElementsCheckWeiZhongDecode接口-Exception：{}", ioe);
			ErrorUtils.captureException(ioe);
			ret.setCode(Constant.FAIL);
		}
		request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, upstreamCode);
		return ret;
	}

	private boolean validateRequestParams(ChannelTypeHandleResponseObject ret, String returnType,
										  TwoElementsRequestObject requestObject) {
		if (!validateParamNullValue(requestObject)) {
			if (Constant.CUSTOMER_RETURN_ZW.equals(returnType)) {
				ret.setResultCode(Constant.INVALID_PARAMETER_CODE);
				ret.setResultBody(Constant.PARAMETER_NAME_ERROR);
				ret.setResultDesc(Constant.PARAMETER_NAME_ERROR);
			} else {
				ret.setResultCode(Constant.JK_RETURN_CODE_INVALID_PARAMETER);
				ret.setResultBody(Constant.PARAMETER_NAME_ERROR);
				ret.setResultDesc(Constant.PARAMETER_NAME_ERROR);
			}
			return false;
		} else if (StringUtils.isBlank(requestObject.getName()) ||
				!PatternTools.checkResult(Constant.PATTERN_CN_NAME_REGEX, requestObject.getName())) {
			if (Constant.CUSTOMER_RETURN_ZW.equals(returnType)) {
				ret.setResultCode(Constant.NAME_FORMAT_ERROR_CODE);
				ret.setResultDesc(Constant.NAME_FORMAT_ERROR);
				ret.setResultBody(Constant.NAME_FORMAT_ERROR);
			} else {
				ret.setResultCode(Constant.JK_NAME_FORMAT_ERROR_CODE);
				ret.setResultDesc(Constant.JK_NAME_FORMAT_ERROR);
				ret.setResultBody(Constant.JK_NAME_FORMAT_ERROR);
			}
			return false;
		} else if (StringUtils.isBlank(requestObject.getIdCode())
				|| !PatternTools.checkResult(Constant.PATTERN_ID_NUMBER_REGEX, requestObject.getIdCode())) {
			if (Constant.CUSTOMER_RETURN_ZW.equals(returnType)) {
				ret.setResultCode(Constant.IDCODE_FORMAT_ERROR_CODE);
				ret.setResultDesc(Constant.IDCODE_FORMAT_ERROR);
				ret.setResultBody(Constant.IDCODE_FORMAT_ERROR);
			} else {
				ret.setResultCode(Constant.JK_IDCODE_FORMAT_ERROR_CODE);
				ret.setResultDesc(Constant.JK_IDCODE_FORMAT_ERROR);
				ret.setResultBody(Constant.JK_IDCODE_FORMAT_ERROR);
			}
			return false;
		} else {
			return true;
		}
	}

	private boolean validateParamNullValue(Object jsonResolveObject) {
		ValidatorFactory vf = Validation.buildDefaultValidatorFactory();
		Validator validator = vf.getValidator();
		Set<ConstraintViolation<Object>> set = validator.validate(jsonResolveObject);
		for (ConstraintViolation<Object> con : set) {
			if (StringUtils.isNotBlank(con.getMessage())) {
				return false;
			}
		}
		return true;
	}

	private void setTwoElementsCheckResponse(ChannelTypeHandleResponseObject ret, String ifJkCode, IdentityCardCheckResponseObject responseObject) {
		if ("0".equals(responseObject.getCode())) {
			// 查询成功
			if(responseObject.getData() != null){
				if("1".equals(responseObject.getData().getCompStatus())){
					ret.setIsCharge(Constant.IS_CHARGE);
					ret.setResultCode(Constant.CUSTOMER_RETURN_JK.equals(ifJkCode) ? Constant.JK_RETURN_CODE_MATCH : Constant.MATCH_CODE);
					ret.setResultBody(Constant.MATCH);
					ret.setResultDesc(Constant.MATCH);

				}else if("2".equals(responseObject.getData().getCompStatus())){
					ret.setIsCharge(Constant.IS_CHARGE);
					ret.setResultCode(Constant.CUSTOMER_RETURN_JK.equals(ifJkCode) ? Constant.JK_RETURN_CODE_NO_MATCH : Constant.UN_MATCH_CODE);
					ret.setResultBody(Constant.UN_MATCH);
					ret.setResultDesc(Constant.UN_MATCH);
				}else if ("3".equals(responseObject.getData().getCompStatus())){
					ret.setIsCharge(Constant.IS_NOT_CHARGE);
					ret.setResultCode(Constant.CUSTOMER_RETURN_JK.equals(ifJkCode) ? Constant.JK_RETURN_CODE_NO_INFO : Constant.UN_FIND_NUM_CODE);
					ret.setResultBody(Constant.UN_FIND_NUM);
					ret.setResultDesc(Constant.UN_FIND_NUM);
				}else{
					ret.setIsCharge(Constant.IS_NOT_CHARGE);
					ret.setResultCode(Constant.CUSTOMER_RETURN_JK.equals(ifJkCode) ? Constant.JK_RETURN_CODE_OTHER_ERROR : Constant.OTHER_ERROR_CODE);
					ret.setResultBody(Constant.OTHER_ERROR);
					ret.setResultDesc(Constant.OTHER_ERROR);
				}
			}

		}else if ("200".equals(responseObject.getCode())) {
			ret.setIsCharge(Constant.IS_NOT_CHARGE);
			ret.setResultCode(Constant.CUSTOMER_RETURN_JK.equals(ifJkCode) ? Constant.JK_RETURN_CODE_NO_INFO : Constant.UN_FIND_NUM_CODE);
			ret.setResultBody(Constant.UN_FIND_NUM);
			ret.setResultDesc(Constant.UN_FIND_NUM);
		}else if("205".equals(responseObject.getCode())){
			ret.setIsCharge(Constant.IS_NOT_CHARGE);
			ret.setResultCode(Constant.CUSTOMER_RETURN_JK.equals(ifJkCode) ? Constant.JK_RETURN_CODE_INVALID_PARAMETER : Constant.INVALID_PARAMETER_CODE);
			ret.setResultBody(Constant.INVALID_PARAMETER);
			ret.setResultDesc(Constant.INVALID_PARAMETER);
		}else{
			// 其他错误
			ret.setIsCharge(Constant.IS_NOT_CHARGE);
			ret.setResultCode(Constant.CUSTOMER_RETURN_JK.equals(ifJkCode) ? Constant.JK_RETURN_CODE_OTHER_ERROR : Constant.OTHER_ERROR_CODE);
			ret.setResultBody(Constant.OTHER_ERROR);
			ret.setResultDesc(Constant.OTHER_ERROR);
		}
	}
}
