package info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import info.aspirecn.iov.sjjh.commons.lang.ChannelTypeHandleResponseObject;
import info.aspirecn.iov.sjjh.commons.lang.Constant;
import info.aspirecn.iov.sjjh.commons.lang.PatternTools;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.service.TelecomOnlineStatusService;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.utils.Utils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.utils.YHProperties;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.vo.TelecomOnlineResObject;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh_interface.vo.TelecomCustomBody;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.concurrent.TimeUnit;

/**
 * @author jianggang
 * @Date 2019/6/11 09:43:22
 * @Description
 */

@Slf4j
@Service
public class TelecomOnlineStatusServiceImpl implements TelecomOnlineStatusService {

    @Autowired
    private YHProperties yhProperties;

    private OkHttpClient okHttpClient = new OkHttpClient.Builder().build();

    @Override
    public ChannelTypeHandleResponseObject callTelecomOnlineStatus(HttpServletRequest request, String channelId, String ifJkCode, Integer outTime, String customBody) {
        ChannelTypeHandleResponseObject res = new ChannelTypeHandleResponseObject();

        /**默认使用卓望返回码*/
        if(StringUtils.isBlank(ifJkCode)){
            ifJkCode = Constant.CUSTOMER_RETURN_ZW;
        }
        /**设置默认超时时间*/
        if (outTime == null || outTime <= 0) {
            outTime = Constant.HTTPCLIENT_CONNECTTIMEOUT;
        }

        ObjectMapper mapper = new ObjectMapper();
        TelecomCustomBody telecomCustomBody = null;
        /**参数校验*/
        try {
            telecomCustomBody = mapper.readValue(customBody,TelecomCustomBody.class);
        } catch (IOException e) {
            log.info("----------->电信在网状态:参数错误：{}",e);
            ErrorUtils.captureException(e);
            res.setCode(Constant.SUCCESS);
            res.setIsCharge(Constant.IS_NOT_CHARGE);
            /**参数错误*/
            if(Constant.CUSTOMER_RETURN_JK.equals(ifJkCode)){
                res.setResultCode(Constant.JK_RETURN_CODE_INVALID_PARAMETER);
            }else{
                res.setResultCode(Constant.INVALID_PARAMETER_CODE);
            }
            res.setResultBody(Constant.PARAMETER_NAME_ERROR);
        	res.setResultDesc(Constant.PARAMETER_NAME_ERROR);
            return res;
        }
        try {
        /**手机号码格式不正确*/
            if(!PatternTools.checkResult(0,telecomCustomBody.getPhone())){
                if(Constant.CUSTOMER_RETURN_ZW.equals(ifJkCode)) {
                    res.setResultCode(Constant.MOBILE_FORMAT_ERROR_CODE);
                    res.setResultDesc(Constant.MOBILE_FORMAT_ERROR);
                    res.setResultBody(Constant.MOBILE_FORMAT_ERROR);
                } else {
                    res.setResultCode(Constant.JK_MOBILE_FORMAT_ERROR_CODE);
                    res.setResultDesc(Constant.JK_MOBILE_FORMAT_ERROR);
                    res.setResultBody(Constant.JK_MOBILE_FORMAT_ERROR);
                }
                res.setCode(Constant.SUCCESS);
                res.setIsCharge(Constant.IS_NOT_CHARGE);
                return res;
            }
            OkHttpClient client = okHttpClient.newBuilder()
                    .connectTimeout(outTime,TimeUnit.MILLISECONDS)
                    .writeTimeout(outTime,TimeUnit.MILLISECONDS)
                    .readTimeout(outTime,TimeUnit.MILLISECONDS)
                    .build();
            String paramsString = "apiKey="+yhProperties.getApiKeyY002c1()
                    +"&username="+yhProperties.getUsername()
                    +"&format=json&params="+ Utils.encrypt(yhProperties.getKey(),"phone="+telecomCustomBody.getPhone());
            log.info("---------------->发送参数为：{}", paramsString);
            RequestBody requestBody = RequestBody.create(MediaType.parse("application/json"),"");
            Request okRequest = new Request.Builder()
                    .url(yhProperties.getUri()+"?"+paramsString)
                    .post(requestBody)
                    .build();
        Response response = null;
        response = client.newCall(okRequest).execute();
        String responseJson = response.body().string();
            log.info("----------------->上游接口返回数据为：{}",responseJson);
            TelecomOnlineResObject telecomOnlineResObject = mapper.readValue(responseJson,TelecomOnlineResObject.class);
            /**判断计费状态*/
            boolean isCharge = Constant.TELECOM_RES_CODE_ZERO.equals(telecomOnlineResObject.getCode())
                    &&(Constant.TELECOM_RES_RESULT_ZERO.equals(telecomOnlineResObject.getData().getResult())
                    || Constant.TELECOM_RES_RESULT_ONE.equals(telecomOnlineResObject.getData().getResult())
                    || Constant.TELECOM_RES_RESULT_TWO.equals(telecomOnlineResObject.getData().getResult())
                    || Constant.TELECOM_RES_RESULT_THREE.equals(telecomOnlineResObject.getData().getResult()));
            if(isCharge){
                res.setIsCharge(Constant.IS_CHARGE);
            }else{
                res.setIsCharge(Constant.IS_NOT_CHARGE);
            }
            /**设置请求状态*/
            res.setCode(Constant.SUCCESS);
            switchResultCode(ifJkCode,telecomOnlineResObject,res);
            /**设置上游返回码日志记录*/
            request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE,telecomOnlineResObject.getCode());
        } catch (IOException e) {
            log.info("-------->电信在网状态-上游接口调用失败",e);
            ErrorUtils.captureException(e);
            res.setCode(Constant.FAIL);
            res.setIsCharge(Constant.IS_NOT_CHARGE);
            return res;
        }
        return res;
    }


    public void switchResultCode(String ifJkCode, TelecomOnlineResObject telecomOnlineTimeResObject, ChannelTypeHandleResponseObject res) {
        /**jk码*/
        if (Constant.CUSTOMER_RETURN_JK.equals(ifJkCode)) {
            if (Constant.TELECOM_RES_CODE_ZERO.equals(telecomOnlineTimeResObject.getCode())) {
                switch (telecomOnlineTimeResObject.getData().getResult()) {
                    case "0":
                        res.setResultCode(Constant.JK_RETURN_CODE_1110);
                        res.setResultBody(Constant.RETURN_MESSAGE_64);
                        res.setResultDesc(Constant.RETURN_MESSAGE_64);
                        break;
                    case "1":
                        res.setResultCode(Constant.JK_RETURN_CODE_1111);
                        res.setResultBody(Constant.RETURN_MESSAGE_65);
                        res.setResultDesc(Constant.RETURN_MESSAGE_65);
                        break;
                    case "2":
                        res.setResultCode(Constant.JK_RETURN_CODE_1112);
                        res.setResultBody(Constant.RETURN_MESSAGE_66);
                        res.setResultDesc(Constant.RETURN_MESSAGE_66);
                        break;
                    case "3":
                        res.setResultCode(Constant.JK_RETURN_CODE_1113);
                        res.setResultBody(Constant.RETURN_MESSAGE_67);
                        res.setResultDesc(Constant.RETURN_MESSAGE_67);
                        break;
                    case "N":
                        res.setResultCode(Constant.JK_RETURN_CODE_1099);
                        res.setResultBody(Constant.RETURN_MESSAGE_60);
                        res.setResultDesc(Constant.RETURN_MESSAGE_60);
                        break;
                    case "4":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    default:
                        break;
                }
            } else {
                switch (telecomOnlineTimeResObject.getCode()) {
                    case "200":
                        res.setResultCode(Constant.JK_RETURN_CODE_1099);
                        res.setResultBody(Constant.RETURN_MESSAGE_60);
                        res.setResultDesc(Constant.RETURN_MESSAGE_60);
                        break;
                    case "201":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "202":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "203":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "204":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "206":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "208":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "209":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "210":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "211":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "260":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "261":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "262":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "263":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "500":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "501":
                        res.setResultCode(Constant.JK_RETURN_CODE_9901);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "205":
                        res.setResultCode(Constant.JK_RETURN_CODE_9909);
                        res.setResultBody(Constant.RETURN_MESSAGE_77);
                        res.setResultDesc(Constant.RETURN_MESSAGE_77);
                        break;
                    case "207":
                        res.setResultCode(Constant.JK_RETURN_CODE_9922);
                        res.setResultBody(Constant.RETURN_MESSAGE_62);
                        res.setResultDesc(Constant.RETURN_MESSAGE_62);
                        break;
                    case "908":
                        res.setResultCode(Constant.JK_RETURN_CODE_9903);
                        res.setResultBody(Constant.RETURN_MESSAGE_63);
                        res.setResultDesc(Constant.RETURN_MESSAGE_63);
                        break;
                    default:
                        break;
                }
            }
        }
        /**卓望码*/
        else {
            if (Constant.TELECOM_RES_CODE_ZERO.equals(telecomOnlineTimeResObject.getCode())) {
                switch (telecomOnlineTimeResObject.getData().getResult()) {
                    case "0":
                        res.setResultCode(Constant.ZW_RETURN_CODE_64);
                        res.setResultBody(Constant.RETURN_MESSAGE_64);
                        res.setResultDesc(Constant.RETURN_MESSAGE_64);
                        break;
                    case "1":
                        res.setResultCode(Constant.ZW_RETURN_CODE_65);
                        res.setResultBody(Constant.RETURN_MESSAGE_65);
                        res.setResultDesc(Constant.RETURN_MESSAGE_65);
                        break;
                    case "2":
                        res.setResultCode(Constant.ZW_RETURN_CODE_66);
                        res.setResultBody(Constant.RETURN_MESSAGE_66);
                        res.setResultDesc(Constant.RETURN_MESSAGE_66);
                        break;
                    case "3":
                        res.setResultCode(Constant.ZW_RETURN_CODE_67);
                        res.setResultBody(Constant.RETURN_MESSAGE_67);
                        res.setResultDesc(Constant.RETURN_MESSAGE_67);
                        break;
                    case "N":
                        res.setResultCode(Constant.ZW_RETURN_CODE_60);
                        res.setResultBody(Constant.RETURN_MESSAGE_60);
                        res.setResultDesc(Constant.RETURN_MESSAGE_60);
                        break;
                    case "4":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    default:
                        break;
                }
            } else {
                switch (telecomOnlineTimeResObject.getCode()) {
                    case "200":
                        res.setResultCode(Constant.ZW_RETURN_CODE_60);
                        res.setResultBody(Constant.RETURN_MESSAGE_60);
                        res.setResultDesc(Constant.RETURN_MESSAGE_60);
                        break;
                    case "201":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "202":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "203":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "204":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "206":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "208":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "209":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "210":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "211":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "260":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "261":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "262":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "263":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "500":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "501":
                        res.setResultCode(Constant.ZW_RETURN_CODE_61);
                        res.setResultBody(Constant.RETURN_MESSAGE_61);
                        res.setResultDesc(Constant.RETURN_MESSAGE_61);
                        break;
                    case "205":
                        res.setResultCode(Constant.ZW_RETURN_CODE_77);
                        res.setResultBody(Constant.RETURN_MESSAGE_77);
                        res.setResultDesc(Constant.RETURN_MESSAGE_77);
                        break;
                    case "207":
                        res.setResultCode(Constant.ZW_RETURN_CODE_62);
                        res.setResultBody(Constant.RETURN_MESSAGE_62);
                        res.setResultDesc(Constant.RETURN_MESSAGE_62);
                        break;
                    case "908":
                        res.setResultCode(Constant.ZW_RETURN_CODE_63);
                        res.setResultBody(Constant.RETURN_MESSAGE_63);
                        res.setResultDesc(Constant.RETURN_MESSAGE_63);
                        break;
                    default:
                        break;
                }
            }
        }
    }
}
