package info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import info.aspirecn.iov.sjjh.commons.lang.ChannelTypeHandleResponseObject;
import info.aspirecn.iov.sjjh.commons.lang.Constant;
import info.aspirecn.iov.sjjh.commons.lang.PatternTools;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.constant.SjjhConstant;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.service.VehicleViolationInfoService;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.utils.Utils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.vo.VehicleViolationInfoJsonResolveObject;
import lombok.extern.slf4j.Slf4j;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import java.net.SocketTimeoutException;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class VehicleViolationInfoServiceImpl implements VehicleViolationInfoService {
    @Value("${yh.uri}")
    private String yhUrl;
    @Value("${yh.username}")
    private String yhUserName;
    @Value("${yh.key}")
    private String yhKey;
    @Value("${yh.apiKeyY000v1}")
    private String apiKeyY000v1;
    @Autowired
    private ObjectMapper objectMapper;
    
    private OkHttpClient okHttpClient = new OkHttpClient.Builder().build();

    @Override
    public ChannelTypeHandleResponseObject getVehicleViolationInfo(HttpServletRequest request, String customBody, int outTime) {
        ChannelTypeHandleResponseObject ret = new ChannelTypeHandleResponseObject();
        ret.setIsCharge(Constant.INTERFACE_QUERY_NO_FEE);
        String upstreamCode = Constant.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE;
        String returnType = request.getHeader(Constant.CUSTOMER_RETURN_TYPE_PARA);

        if (StringUtils.isBlank(returnType)) {
            returnType = Constant.CUSTOMER_RETURN_ZW;
        }
        if (outTime <= 0) {
            outTime = Constant.HTTPCLIENT_CONNECTTIMEOUT;
        }
        log.info("getVehicleViolationInfo---customBody={},outTime={}", customBody, outTime);

        try {
            VehicleViolationInfoJsonResolveObject jsonResolveObject = objectMapper.readValue(customBody, VehicleViolationInfoJsonResolveObject.class);
            //参数校验
            if (!validateParams(ret, returnType, jsonResolveObject)) {
                ret.setCode(Constant.SUCCESS);
                request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, upstreamCode);
                return ret;
            }

            String params = Utils.encrypt(yhKey, jsonResolveObject.toString());
            FormBody.Builder formBodyBuilder = new FormBody.Builder()
                    .add("apiKey", apiKeyY000v1)
                    .add("username", yhUserName)
                    .add("params", params);
            Request okRequest = new Request.Builder()
                    .post(formBodyBuilder.build())
                    .url(yhUrl)
                    .build();
            OkHttpClient client = okHttpClient.newBuilder()
                    .connectTimeout(outTime, TimeUnit.MILLISECONDS)
                    .readTimeout(outTime, TimeUnit.MILLISECONDS)
                    .writeTimeout(outTime, TimeUnit.MILLISECONDS)
                    .build();
            Response response = client.newCall(okRequest).execute();
            String responseContext = "";
            if (response.body() != null) {
                responseContext = response.body().string();
            }
            log.info("supplier10000003.getVehicleViolationInfo接口responseContext={}", responseContext);

            JSONObject responseJsonObject = JSONObject.parseObject(responseContext);
            log.info("supplier10000003.getVehicleViolationInfo接口responseObject={}", responseJsonObject);

            int httpResponseCode = response.code();
            log.info("supplier10000003.getVehicleViolationInfo接口httpResponseCode={}", httpResponseCode);

            if (responseJsonObject != null) {
                ret.setCode(Constant.SUCCESS);
                setVehicleViolationInfoResponse(ret, returnType, responseJsonObject);
                upstreamCode = responseJsonObject.get("code").toString();
                response.close();
            } else {
                ret.setCode(Constant.FAIL);
            }
        } catch (SocketTimeoutException ste) {
            ErrorUtils.captureException(ste);
            log.info("supplier10000003.getVehicleViolationInfo接口SocketTimeoutException={}", ste);
            ret.setCode(Constant.REQUEST_TIMEOUT);
        } catch (Exception ioe) {
            ErrorUtils.captureException(ioe);
            log.info("supplier10000003.getVehicleViolationInfo接口Exception={}", ioe);
            ret.setCode(Constant.FAIL);
        }

        //上游返回码
        request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, upstreamCode);
        return ret;
    }

    private boolean validateParams(ChannelTypeHandleResponseObject ret, String returnType,
                                   VehicleViolationInfoJsonResolveObject jsonResolveObject) {
        if (!validateParamNullValue(jsonResolveObject)) {
            if (Constant.CUSTOMER_RETURN_ZW.equals(returnType)) {
                ret.setResultCode(Constant.INVALID_PARAMETER_CODE);
                ret.setResultBody(Constant.PARAMETER_NAME_ERROR);
                ret.setResultDesc(Constant.PARAMETER_NAME_ERROR);
            } else {
                ret.setResultCode(Constant.JK_RETURN_CODE_INVALID_PARAMETER);
                ret.setResultBody(Constant.PARAMETER_NAME_ERROR);
                ret.setResultDesc(Constant.PARAMETER_NAME_ERROR);
            }
            return false;
        } else if (StringUtils.isBlank(jsonResolveObject.getCarNumber())
                || StringUtils.isBlank(jsonResolveObject.getCarCode())
                || StringUtils.isBlank(jsonResolveObject.getCarDriverNumber())
                || StringUtils.isBlank(jsonResolveObject.getCarType())
                || !PatternTools.checkResult(Constant.PATTERN_CAR_NUMBER_REGEX, jsonResolveObject.getCarNumber())
                || !ArrayUtils.contains(SjjhConstant.VEHICLE_VIOLATION_CAR_TYPE, jsonResolveObject.getCarType())) {
            if (Constant.CUSTOMER_RETURN_ZW.equals(returnType)) {
                ret.setResultCode(Constant.INVALID_PARAMETER_CODE);
                ret.setResultBody(Constant.INVALID_PARAMETER);
                ret.setResultDesc(Constant.INVALID_PARAMETER);
            } else {
                ret.setResultCode(Constant.JK_RETURN_CODE_INVALID_PARAMETER);
                ret.setResultBody(Constant.INVALID_PARAMETER);
                ret.setResultDesc(Constant.INVALID_PARAMETER);
            }
            return false;
        } else {
            return true;
        }
    }

    private void setVehicleViolationInfoResponse(ChannelTypeHandleResponseObject ret, String returnType, JSONObject jsonObject) {
        String responseCode = jsonObject.get("code").toString();
        boolean isCharge = !StringUtils.isBlank(responseCode)
                && ArrayUtils.contains(SjjhConstant.VEHICLE_VIOLATION_CHARGE_CODE, responseCode);

        if (isCharge) {
            ret.setIsCharge(Constant.INTERFACE_QUERY_FEE);
        }

        if (Constant.CUSTOMER_RETURN_JK.equals(returnType)) {
            if (StringUtils.isNotBlank(responseCode) && responseCode.equals(SjjhConstant.VEHICLE_VIOLATION_RESPONSE_CODE_0)) {
                // 查询成功
                ret.setResultCode(Constant.JK_RETURN_CODE_MATCH);
                ret.setResultBody(JSONObject.toJSONString(jsonObject.get("data")));
                ret.setResultDesc(SjjhConstant.QUERY_SUCCESS);
            } else if (StringUtils.isNotBlank(responseCode) && responseCode.equals(SjjhConstant.VEHICLE_VIOLATION_RESPONSE_CODE_200)) {
                // 库无
                ret.setResultCode(Constant.JK_RETURN_CODE_NO_INFO);
                ret.setResultBody(Constant.UN_FIND_NUM);
                ret.setResultDesc(Constant.UN_FIND_NUM);
            } else if (StringUtils.isNotBlank(responseCode) && responseCode.equals(SjjhConstant.VEHICLE_VIOLATION_RESPONSE_CODE_910)) {
                // 输入车牌号或车架号或发动机号有误
                ret.setResultCode(Constant.JK_RETURN_CODE_1150);
                ret.setResultBody(Constant.RETURN_MESSAGE_80);
                ret.setResultDesc(Constant.RETURN_MESSAGE_80);
            } else if (StringUtils.isNotBlank(responseCode) && (responseCode.equals(SjjhConstant.VEHICLE_VIOLATION_RESPONSE_CODE_205)
                    || responseCode.equals(SjjhConstant.VEHICLE_VIOLATION_RESPONSE_CODE_913))) {
                // 参数错误
                ret.setResultCode(Constant.JK_RETURN_CODE_INVALID_PARAMETER);
                ret.setResultBody(Constant.INVALID_PARAMETER);
                ret.setResultDesc(Constant.INVALID_PARAMETER);
            } else {
                // 查询错误
                ret.setResultCode(Constant.JK_RETURN_CODE_OTHER_ERROR);
                ret.setResultBody(Constant.OTHER_ERROR);
                ret.setResultDesc(Constant.OTHER_ERROR);
            }
        } else {
            if (StringUtils.isNotBlank(responseCode) && responseCode.equals(SjjhConstant.VEHICLE_VIOLATION_RESPONSE_CODE_0)) {
                // 查询成功
                ret.setResultCode(Constant.MATCH_CODE);
                ret.setResultBody(JSONObject.toJSONString(jsonObject.get("data")));
                ret.setResultDesc(SjjhConstant.QUERY_SUCCESS);
            } else if (StringUtils.isNotBlank(responseCode) && responseCode.equals(SjjhConstant.VEHICLE_VIOLATION_RESPONSE_CODE_200)) {
                // 库无
                ret.setResultCode(Constant.UN_FIND_NUM_CODE);
                ret.setResultBody(Constant.UN_FIND_NUM);
                ret.setResultDesc(Constant.UN_FIND_NUM);
            } else if (StringUtils.isNotBlank(responseCode) && responseCode.equals(SjjhConstant.VEHICLE_VIOLATION_RESPONSE_CODE_910)) {
                // 输入车牌号或车架号或发动机号有误
                ret.setResultCode(Constant.ZW_RETURN_CODE_80);
                ret.setResultBody(Constant.RETURN_MESSAGE_80);
                ret.setResultDesc(Constant.RETURN_MESSAGE_80);
            } else if (StringUtils.isNotBlank(responseCode) && (responseCode.equals(SjjhConstant.VEHICLE_VIOLATION_RESPONSE_CODE_205)
                    || responseCode.equals(SjjhConstant.VEHICLE_VIOLATION_RESPONSE_CODE_913))) {
                // 参数错误
                ret.setResultCode(Constant.INVALID_PARAMETER_CODE);
                ret.setResultBody(Constant.INVALID_PARAMETER);
                ret.setResultDesc(Constant.INVALID_PARAMETER);
            } else {
                // 查询错误
                ret.setResultCode(Constant.OTHER_ERROR_CODE);
                ret.setResultBody(Constant.OTHER_ERROR);
                ret.setResultDesc(Constant.OTHER_ERROR);
            }
        }
    }

    private boolean validateParamNullValue(VehicleViolationInfoJsonResolveObject jsonResolveObject) {
        ValidatorFactory vf = Validation.buildDefaultValidatorFactory();
        Validator validator = vf.getValidator();
        Set<ConstraintViolation<VehicleViolationInfoJsonResolveObject>> set = validator.validate(jsonResolveObject);
        for (ConstraintViolation<VehicleViolationInfoJsonResolveObject> con : set) {
            if (StringUtils.isNotBlank(con.getMessage())) {
                return false;
            }
        }
        return true;
    }
}
