package info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import info.aspirecn.iov.sjjh.commons.lang.ChannelTypeHandleResponseObject;
import info.aspirecn.iov.sjjh.commons.lang.Constant;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.service.ForeignInvestService;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.utils.ForeignInvestProperties;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.utils.Utils;
import info.aspirecn.rdc.iov.sjjh.servicenode.supplier.yh.vo.ForeignInvestResponseObject;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class ForeignInvestServiceImpl implements ForeignInvestService {

	@Autowired
	private ForeignInvestProperties foreignInvestProperties; 
    @Autowired
    ObjectMapper mapper;
    
    private OkHttpClient client = new OkHttpClient.Builder().build();
    
    @Override
    public ChannelTypeHandleResponseObject foreignInvest(HttpServletRequest request,
                                                         Integer outTime,
                                                         String customBody) {

        ChannelTypeHandleResponseObject res = new ChannelTypeHandleResponseObject();
        res.setIsCharge(Constant.IS_NOT_CHARGE);
        /**设置默认超时时间*/
        if (outTime == null || outTime.intValue() <= 0) {
            outTime = Constant.HTTPCLIENT_CONNECTTIMEOUT;
        }
        try {
            Map map = mapper.readValue(customBody, Map.class);
            OkHttpClient okHttpClient = client.newBuilder()
                    .connectTimeout(outTime, TimeUnit.MILLISECONDS)
                    .writeTimeout(outTime, TimeUnit.MILLISECONDS)
                    .readTimeout(outTime, TimeUnit.MILLISECONDS)
                    .build();
            if(map.get("cname") == null){
            	log.error("cname为空");
            	res.setCode(Constant.SUCCESS);
            	res.setResultCode(Constant.INVALID_PARAMETER_CODE);
            	res.setResultBody(Constant.INVALID_PARAMETER);
            }
            String paramString = "cname=" + map.get("cname");
            String encryptParamString = Utils.encrypt(foreignInvestProperties.getApiSecret(), paramString);
            log.info("foreignInvestProperties:{}",foreignInvestProperties);
            RequestBody requestBody = new FormBody.Builder().add("apiKey", foreignInvestProperties.getApiKey())
                    .add("username", foreignInvestProperties.getUsername())
                    .add("format", "json")
                    .add("params", encryptParamString)
                    .build();
            log.info("requestBody:{}",requestBody);
            Request okRequest = new Request.Builder()
                    .url(foreignInvestProperties.getUrl())
                    .post(requestBody)
                    .build();
            Response response = okHttpClient.newCall(okRequest).execute();
            if(!response.isSuccessful()){
            	throw new Exception("request status is not 200,http status:"+String.valueOf(response.code()));
            }
            String responseJson = response.body().string();
            log.info("--------------->接口返回数据为：{}", responseJson);
            ForeignInvestResponseObject foreignInvestResponseObject = mapper.readValue(responseJson, ForeignInvestResponseObject.class);
            /**判断计费状态*/
            boolean isCharge = Constant.INFO_CODE_ZERO.equals(foreignInvestResponseObject.getCode());
            if (isCharge) {
                res.setIsCharge(Constant.IS_CHARGE);
            } else {
                res.setIsCharge(Constant.IS_NOT_CHARGE);
            }
            res.setCode(Constant.SUCCESS);
            if(Constant.INFO_CODE_ZERO.equals(foreignInvestResponseObject.getCode())){
            	res.setResultCode(Constant.SUCCESS);
            	String value = mapper.writeValueAsString(foreignInvestResponseObject.getData());  
            	res.setResultBody(value);
            }else if(Constant.INFO_PARAM_ERROR_202.equals(foreignInvestResponseObject.getCode())
            		||Constant.INFO_PARAM_ERROR_203.equals(foreignInvestResponseObject.getCode())
            		||Constant.INFO_PARAM_ERROR_205.equals(foreignInvestResponseObject.getCode())){
            	res.setResultCode(Constant.INVALID_PARAMETER_CODE);
            	res.setResultBody(Constant.INVALID_PARAMETER);
            }else {
            	res.setResultCode(Constant.OTHER_ERROR_CODE);
            	res.setResultBody(Constant.OTHER_ERROR);
            }
            /**设置上游返回码日志记录*/
            request.setAttribute(Constant.LOG_UPSTREAM_RESPONSE_CODE, foreignInvestResponseObject.getCode());
        } catch (IOException e) {
        	ErrorUtils.captureException(e);
        	log.error("IOException,message:{}",e);
            res.setCode(Constant.FAIL);
            res.setIsCharge(Constant.IS_NOT_CHARGE);
            return res;
        }catch (Exception e) {
        	ErrorUtils.captureException(e);
        	log.error("Exception,message:{}",e);
            res.setCode(Constant.FAIL);
            res.setIsCharge(Constant.IS_NOT_CHARGE);
            return res;
        }
        return res;
    }

}